import { css } from 'lit'

export default css`
    :host {
        display: inline-block;
        width: 1em;
        height: 1em;
        box-sizing: content-box !important;
    }

    svg {
        display: block;
        height: 100%;
        width: 100%;
    }
`
