import { css } from 'lit'

export default css`
    :host {
        display: block;
    }

    sl-skeleton {
        --border-radius: 4px;
        --color: hsl(240 5.9% 50%);
        --sheen-color: hsl(240 5.9% 60%);

        margin-bottom: 10px;
    }
`
